package com.jsonpfy.webservices;

import java.io.IOException;

import javax.servlet.http.HttpServlet;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import org.json.JSONArray;

import com.google.appengine.api.datastore.Query.FilterOperator;
import com.jsonpfy.operations.ListData;
import com.jsonpfy.utilities.CallbackUtilities;
import com.jsonpfy.utilities.JsonpfyProperties;
import com.jsonpfy.utilities.URLUtilities;

@SuppressWarnings("serial")
public class ListDataServlet extends HttpServlet {

	// http://localhost:8888/ListDataService
	// ?kind=Comment
	// &filterField=projectID&filterValue=Projectnection
	// &filterField=classPath&filterValue=com.projectnection.client.Entities.EntityPage
	// &sortField=date&sortDirection=desc

	public void doGet(HttpServletRequest request, HttpServletResponse response)
			throws IOException {

		if (JsonpfyProperties.OPEN) {

			response.getWriter().println(listData(request));
		}
	}

	public void doPost(HttpServletRequest request, HttpServletResponse response)
			throws IOException {

		response.getWriter().println(listData(request));
	}

	private String listData(HttpServletRequest request) {

		String answer = null;

		if (JsonpfyProperties.OPEN || JsonpfyProperties.authenticate(request)) {

			JSONArray entities = null;

			String kind = URLUtilities.decode(request
					.getParameter(JsonpfyProperties.ENTITY_KIND));

			String sortField = URLUtilities.decode(request
					.getParameter("sortField"));

			String sortDirection = URLUtilities.decode(request
					.getParameter("sortDirection"));

			String returnField = URLUtilities.decode(request
					.getParameter("returnField"));

			entities = getFilter(request, kind, sortField, sortDirection,
					returnField);

			answer = CallbackUtilities.getCallback(
					request.getParameter("callback"), entities.toString());

		}

		return answer;
	}

	private JSONArray getFilter(HttpServletRequest request, String kind,
			String sortField, String sortDirection, final String returnField) {

		JSONArray entities = null;

		String filterField = URLUtilities.decode(request
				.getParameter("filterField"));

		if (filterField != null) {

			String filterValue = URLUtilities.decode(request
					.getParameter("filterValue"));

			if (filterValue != null) {

				entities = ListData.jsonListFilter(kind, filterField,
						filterValue, sortField, sortDirection, returnField,
						FilterOperator.EQUAL);
			} else {

				String filterBiggerValue = URLUtilities.decode(request
						.getParameter("filterBiggerValue"));

				entities = ListData.jsonListFilter(kind, filterField,
						filterBiggerValue, sortField, sortDirection,
						returnField, FilterOperator.GREATER_THAN);
			}

		} else {

			entities = getMultipleFilters(request, kind, sortField,
					sortDirection, returnField);
		}

		return entities;
	}

	private JSONArray getMultipleFilters(HttpServletRequest request,
			String kind, String sortField, String sortDirection,
			final String returnField) {

		JSONArray entities = null;

		String filterField1 = URLUtilities.decode(request
				.getParameter("filterField1"));

		if (filterField1 != null) {

			String filterField2 = URLUtilities.decode(request
					.getParameter("filterField2"));

			String filterField3 = URLUtilities.decode(request
					.getParameter("filterField3"));

			String filterField4 = URLUtilities.decode(request
					.getParameter("filterField4"));

			String filterValue1 = URLUtilities.decode(request
					.getParameter("filterValue1"));

			String filterValue2 = URLUtilities.decode(request
					.getParameter("filterValue2"));

			String filterValue3 = URLUtilities.decode(request
					.getParameter("filterValue3"));

			String filterValue4 = URLUtilities.decode(request
					.getParameter("filterValue4"));

			entities = ListData.jsonMultipleListFilter(kind, filterField1,
					filterField2, filterField3, filterField4, filterValue1,
					filterValue2, filterValue3, filterValue4, sortField,
					sortDirection, returnField);

		} else {

			entities = getKeyFilter(request, kind, sortField, sortDirection,
					returnField);
		}

		return entities;
	}

	private JSONArray getKeyFilter(HttpServletRequest request, String kind,
			String sortField, String sortDirection, final String returnField) {

		JSONArray entities = null;

		String filterKeyField = URLUtilities.decode(request
				.getParameter("filterKeyField"));

		if (filterKeyField != null) {

			String filterKeyKind = URLUtilities.decode(request
					.getParameter("filterKeyKind"));

			String filterKeyValue = URLUtilities.decode(request
					.getParameter("filterKeyValue"));

			entities = ListData.jsonListFilterKey(kind, filterKeyField,
					filterKeyKind, filterKeyValue, sortField, sortDirection,
					returnField);
		} else {

			entities = ListData.jsonList(kind, sortField, sortDirection,
					returnField);
		}

		return entities;
	}
}
